const fs = require('fs');
const path = require('path');

const settingsFilePath = path.join(process.env.APPDATA || (process.platform === 'darwin' ? process.env.HOME + '/Library/Application Support' : process.env.HOME + "/.config"), 'GeminiNative', 'settings.json');
const settingsDirPath = path.dirname(settingsFilePath);

const defaultSettings = {
  autoHideEnabled: true,
  reloadOnPasteEnabled: true,
  isAlwaysOnTop: true,
  isFrameVisible: false,
  launchOnStartup: false,

  currentProvider: "Gemini",
  
  providers: [
    {name: "Gemini", url: "https://gemini.google.com"},
    {name: "ChatGPT", url: "https://chat.openai.com"},
    {name: "Bing Copilot", url: "https://www.bing.com/chat"},
    {name: "Claude", url: "https://claude.ai"},
    {name: "Qwen", url: "https://chat.qwen.ai/"},
    {name: "Perplexity", url: "https://www.perplexity.ai"},
  ]
};

let currentSettings = { ...defaultSettings };

function ensureSettingsDirExists() {
  if (!fs.existsSync(settingsDirPath)) {
    fs.mkdirSync(settingsDirPath, { recursive: true });
  }
}

function loadSettings() {
  ensureSettingsDirExists();
  try {
    if (fs.existsSync(settingsFilePath)) {
      const data = fs.readFileSync(settingsFilePath, 'utf-8');
      const loadedSettings = JSON.parse(data);
      currentSettings = { ...defaultSettings, ...loadedSettings };
    } else {
      // Save default settings if file doesn't exist
      saveSettings();
    }
  } catch (error) {
    console.error('Error loading settings, using defaults:', error);
    currentSettings = { ...defaultSettings };
  }
  return currentSettings;
}

function saveSettings() {
  ensureSettingsDirExists();
  try {
    const data = JSON.stringify(currentSettings, null, 2);
    fs.writeFileSync(settingsFilePath, data, 'utf-8');
  } catch (error) {
    console.error('Error saving settings:', error);
  }
}

// Initialize settings on load
loadSettings();

module.exports = {
  getSetting: (key) => {
    return currentSettings[key];
  },
  updateSetting: (key, value) => {
    if (currentSettings[key] !== value) {
      currentSettings[key] = value;
      saveSettings();
    }
  },
  getAllSettings: () => {
    return { ...currentSettings };
  },
  addProvider: (provider) => {
    currentSettings.providers.push(provider);
    saveSettings();
  },
  deleteProvider: (index) => {
    if (index >= 0 && index < currentSettings.providers.length) {
      currentSettings.providers.splice(index, 1);
      saveSettings();
    }
  },
  updateProvider: (index, provider) => {
    if (index >= 0 && index < currentSettings.providers.length) {
      currentSettings.providers[index] = provider;
      saveSettings();
    }
  },
  getCurrentProviderUrl: () => {
    const currentProviderName = currentSettings.currentProvider;
    const provider = currentSettings.providers.find(p => p.name === currentProviderName);
    return provider ? provider.url : (currentSettings.providers[0] ? currentSettings.providers[0].url : 'https://gemini.google.com'); // Fallback
  }
};
