const { BrowserWindow, screen } = require('electron');
const path = require('path');
const settingsManager = require('./settingsManager');

let mainWindow;
let firstRunWindow;

function positionWindowAtBottomCenter(window) {
  const cursorPoint = screen.getCursorScreenPoint();
  const display = screen.getDisplayNearestPoint(cursorPoint);
  const { workArea } = display;

  const [windowWidth, windowHeight] = window.getSize();

  const x = Math.round(workArea.x + (workArea.width - windowWidth) / 2);
  const y = Math.round(workArea.y + workArea.height - windowHeight - 8);

  window.setPosition(x, y);
}

function createWindow(onCloseCallback, onBlurCallback) {
  const initialWidth = 800;
  const initialHeight = 900;

  mainWindow = new BrowserWindow({
    width: initialWidth,
    height: initialHeight,
    frame: settingsManager.getSetting('isFrameVisible'),
    resizable: false,
    alwaysOnTop: settingsManager.getSetting('isAlwaysOnTop'),
    webPreferences: {
      nodeIntegration: false,
      contextIsolation: true,
    },
    title: 'Gemini',
    icon: path.join(__dirname, 'assets', 'icon.png'),
    skipTaskbar: true,
    show: false
  });

  positionWindowAtBottomCenter(mainWindow);
  // Load the URL of the current provider
  const providerUrl = settingsManager.getCurrentProviderUrl();
  mainWindow.loadURL(providerUrl);
  mainWindow.setMenuBarVisibility(false);

  mainWindow.on('close', onCloseCallback);
  mainWindow.on('closed', () => {
    mainWindow = null;
  });
  mainWindow.on('blur', onBlurCallback);

  return mainWindow;
}

function createFirstRunWindow(appInstance) {
  firstRunWindow = new BrowserWindow({
    width: 800,
    height: 800,
    frame: true,
    resizable: false,
    alwaysOnTop: true,
    webPreferences: {
      nodeIntegration: false,
      contextIsolation: true,
      sandbox: true,
    },
    show: false, 
    skipTaskbar: false, 
    autoHideMenuBar: true, 
    frame: false,
    icon: path.join(__dirname, '..', 'assets', 'icon.png'), // Adjusted path for assets
  });

  firstRunWindow.loadFile(path.join(__dirname, '..', 'firstrun.html')); // Adjusted path for firstrun.html

  firstRunWindow.once('ready-to-show', () => {
    positionWindowAtBottomCenter(firstRunWindow); 
    firstRunWindow.show();
  });

  firstRunWindow.on('closed', () => {
    firstRunWindow = null;
  });
  return firstRunWindow;
}

function getMainWindow() {
    return mainWindow;
}

function toggleWindowVisibility() {
  if (mainWindow) {
    const cursorPoint = screen.getCursorScreenPoint();
    const cursorDisplay = screen.getDisplayNearestPoint(cursorPoint);
    const windowBounds = mainWindow.getBounds();
    const windowDisplay = screen.getDisplayMatching(windowBounds);

    if (mainWindow.isVisible() && mainWindow.isFocused() && windowDisplay.id === cursorDisplay.id) {
      console.log("Hiding main window");
      mainWindow.hide();
    } else {
      console.log("Showing main window");
      if (mainWindow.isMinimized()) {
        mainWindow.restore();
      }
      mainWindow.show();
      mainWindow.focus();
      positionWindowAtBottomCenter(mainWindow);
    }
  } else {
    // This case should ideally be handled by the caller,
    // perhaps by calling createWindow if mainWindow is null.
    // For now, let's log or decide on a consistent behavior.
    console.log("Tried to toggle visibility but mainWindow is not defined.");
  }
}

module.exports = {
  createWindow,
  getMainWindow,
  toggleWindowVisibility,
  positionWindowAtBottomCenter,
  createFirstRunWindow
};
